<?php
/**
 * Created by PhpStorm.
 * User: phillipesilva
 * Date: 01/10/2014
 * Time: 11:58
 */

namespace Geral\Logic\FonteDados;


use Geral\Entities\Paramsqlfontedados;
use Geral\Util\DateUtil;
use Importacao\Logic\Util\ExcelUtils;
use Geral\Entities\Colunafontedados;
use Geral\Entities\Linhafontedados;
use Geral\Util\Ambiente;

class FonteDados {
    /**
     * @param $nrFonteDados      -> Número da Fonte de Dados
     * @param $nrOrg             -> Número da Organização
     * @param $idFonteDados      -> Identificação da Fonte de Dados
     * @param null $comSeparador -> Se tem separado ou não. Utilizado qdo $idFonteDados = 'TPL'
     * @return array
     */
    public static function getParamsValidarFonteDados($nrFonteDados, $nrOrg, $idFonteDados, $comSeparador = null)
    {
        $paramsRet = array();

        if($idFonteDados == 'XLS') {
            $where   = 'NRLINHAFONTEDADOS = ? AND NRORG = ?';
            $orderBy = 'NRPOSICAOINICIAL';

            $linha = Linhafontedados::findOneByNrfontedadosAndNrorg($nrFonteDados, $nrOrg);

            if(is_object($linha)) {
                $params  = array($linha->getNrlinhafontedados(), $nrOrg);
                $colunas = Colunafontedados::getObjectsByWhere($where, $params, $orderBy);

                foreach($colunas as $key => $coluna) {
                    $paramsRet[] = array('LABEL'    => $coluna->getDscolunafontedados(),
                                         'TIPO'     => $coluna->getIdtpcolunafontedados(),
                                         'MASK'     => $coluna->getDsmaskcolunafontedados(),
                                         'REQUIRED' => $coluna->getIdcolunarequerida(),
                                         'FIELD'    => 'FIELD'.$key);
                }
            }
        } else if($idFonteDados == 'TPL') {
            /*
             * Início - Criar array de objetos das Linhas
             *
             * As linhas são cadastradas na tela e serão inseridas no array $linhas de forma ordena, das quais:
             * P - Primeira Linha: linha única.
             * I - Intervalo de Linhas: linhas organizadas de acordo com a posicao inicial cadastrada.
             * U - Última Linha: linha única.
             */
            $priLinha = Linhafontedados::findOneByNrfontedadosAndNrorgAndIdposicaolinha($nrFonteDados, $nrOrg, 'P');
            $ultLinha = Linhafontedados::findOneByNrfontedadosAndNrorgAndIdposicaolinha($nrFonteDados, $nrOrg, 'U');

            $whereLinha = "NRFONTEDADOS = ?
                       AND IDPOSICAOLINHA = ?
                       AND NRORG = ?";

            $paramLinha = array($nrFonteDados, 'I', $nrOrg);
            $orderByLinha = "NRINTERVALOINICIAL";

            $interLinhas = Linhafontedados::getObjectsByWhere($whereLinha, $paramLinha, $orderByLinha);

            if(is_object($priLinha)) {
                $linhas[] = $priLinha;
            }

            foreach ($interLinhas as $interLinha) {
                $linhas[] = $interLinha;
            }

            if(is_object($ultLinha)) {
                $linhas[] = $ultLinha;
            }
            /*
             * Fim - Criar array de objetos das Linhas
             */

            //Cria o array dos parâmetros com as linhas ordenadas e suas respectivas colunas cadastradas.
            $where   = 'NRLINHAFONTEDADOS = ? AND NRORG = ?';
            $orderBy = 'NRPOSICAOINICIAL';

            foreach ($linhas as $keyLine => $linha) {
                $nrLinha = $linha->getNrlinhafontedados();
                $params  = array($nrLinha, $nrOrg);
                $colunas = Colunafontedados::getObjectsByWhere($where, $params, $orderBy);

                foreach($colunas as $keyColum => $coluna) {
                    $paramsRet['LINHAS'] = $linhas;

                    if($comSeparador == 'N') {
                        $paramsRet['COLUNAS'][$nrLinha][] = array('LABEL'    => $coluna->getDscolunafontedados(),
                                                                  'POS_INI'  => $coluna->getNrposicaoinicial(),
                                                                  'POS_FIM'  => $coluna->getNrposicaofinal(),
                                                                  'TIPO'     => $coluna->getIdtpcolunafontedados(),
                                                                  'MASK'     => $coluna->getDsmaskcolunafontedados(),
                                                                  'REQUIRED' => $coluna->getIdcolunarequerida(),
                                                                  'FIELD'    => 'FIELD' . $keyColum);
                    } else {
                        $paramsRet['COLUNAS'][$nrLinha][] = array('LABEL'    => $coluna->getDscolunafontedados(),
                                                                  'TIPO'     => $coluna->getIdtpcolunafontedados(),
                                                                  'MASK'     => $coluna->getDsmaskcolunafontedados(),
                                                                  'REQUIRED' => $coluna->getIdcolunarequerida(),
                                                                  'FIELD'    => 'FIELD' . $keyColum);
                    }
                }
            }
        } else if($idFonteDados == 'SQL') {
            $linha = Linhafontedados::findOneByNrfontedadosAndNrorg($nrFonteDados, $nrOrg);

            if(is_object($linha)) {
                $params = Paramsqlfontedados::findByNrlinhafontedadosAndNrorg($linha->getNrlinhafontedados(), $nrOrg);

                foreach($params as $param) {
                    $paramsRet[] = array('LABEL' => $param->getDsparamsqlfontedados(),
                                         'TIPO'  => $param->getIdtpparamsqlfontedados());
                }
            }
        } else if($idFonteDados == 'TIC'){
            $linhas = Linhafontedados::findByNrfontedadosAndNrorg($nrFonteDados, $nrOrg);

            $where   = 'NRLINHAFONTEDADOS = ? AND NRORG = ?';
            $orderBy = 'NRPOSICAOINICIAL';

            //Cria o array dos parâmetros com as linhas e suas respectivas colunas.
            foreach ($linhas as $linha) {
                $params  = array($linha->getNrlinhafontedados(), $nrOrg);
                $colunas = Colunafontedados::getObjectsByWhere($where, $params, $orderBy);
                $nrLinha = $linha->getNrlinhafontedados();

                foreach($colunas as $keyColum => $coluna) {
                    $paramsRet['LINHAS'] = $linhas;
                    $paramsRet['COLUNAS'][$nrLinha][] = array('LABEL'    => $coluna->getDscolunafontedados(),
                                                              'POS_INI'  => $coluna->getNrposicaoinicial(),
                                                              'POS_FIM'  => $coluna->getNrposicaofinal(),
                                                              'TIPO'     => $coluna->getIdtpcolunafontedados(),
                                                              'MASK'     => $coluna->getDsmaskcolunafontedados(),
                                                              'REQUIRED' => $coluna->getIdcolunarequerida(),
                                                              'FIELD'    => 'FIELD'.$keyColum);
                }
            }
        }

        return $paramsRet;
    }

    /**
     * @param $fileName          -> Nome do arquivo
     * @param $params            -> Parâmetros cadastrados na tela
     * @param $idFonteDados      -> Identificação da Fonte de Dados
     * @param null $comSeparador -> Utilizado qdo $idFonteDados = 'TPL': Se tem separado ou não.
     * @return array|void
     */
    public static function validarFonteDados($fileName, $params, $idFonteDados, $comSeparador = null)
    {
        if($idFonteDados == 'TPL') {
            $dataSource = self::validarTPL($fileName, $params, $comSeparador);
        } else if($idFonteDados == 'XLS') {
            $dataSource = self::validarXLS($fileName, $params);
        }  else if($idFonteDados == 'TIC') {
            $dataSource = self::validarTIC($fileName, $params);
        }

        return $dataSource;
    }

    /*
     * Função que retorna o DataSource(Array de Array's) completo com as linhas do mesmo tipo de validação agrupadados
     * no mesmo item, para que sejam visualizadas juntas no grid de validação.
     */
    private static function validarTPL($fileName, $params, $comSeparador = null)
    {
        $diretorio = realpath('../uploads');
        $file      = $diretorio.DIRECTORY_SEPARATOR.$fileName;

        $fileLinhas = file($file, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);

        //Codificação necessária dos caracteres especiais (ç, á, ...).
        foreach ($fileLinhas as &$fileLinha) {
            $fileLinha = utf8_encode($fileLinha);
        }

        $dataSource = array();

        if($comSeparador == 'S') {
            $separador = end($params)['SEPARADOR'];
            array_pop($params);

            $paramLinhas  = $params['LINHAS'];
            $paramColunas = $params['COLUNAS'];

            foreach($fileLinhas as $keyLinha => $linha) {
                //Primeira Linha
                if ($keyLinha == 0) {
                    foreach ($paramLinhas as $paramLinha) {
                        if ($paramLinha->getIdposicaolinha() == 'P') {
                            $nrLinha = $paramLinha->getNrlinhafontedados();

                            $arrLinha = explode($separador, $linha);
                            $posSeparador = strpos($linha, $separador);

                            self::fillDataSourceSeparador($paramColunas[$nrLinha], $keyLinha, $linha, $dataSource, $nrLinha, $posSeparador, $arrLinha);
                        }
                    }
                }
                //Última Linha
                else if ($keyLinha == (count($fileLinhas) - 1)) {
                    foreach ($paramLinhas as $paramLinha) {
                        if ($paramLinha->getIdposicaolinha() == 'U') {
                            $nrLinha = $paramLinha->getNrlinhafontedados();

                            $arrLinha = explode($separador, $linha);
                            $posSeparador = strpos($linha, $separador);

                            self::fillDataSourceSeparador($paramColunas[$nrLinha], $keyLinha, $linha, $dataSource, $nrLinha, $posSeparador, $arrLinha);
                        }
                    }
                } //Intervalo de Linhas
                else {
                    foreach ($paramLinhas as $paramLinha) {
                        $idPosicao = $paramLinha->getIdposicaolinha();
                        $interFim  = $paramLinha->getNrintervalofinal();
                        $interIni  = $paramLinha->getNrintervaloinicial();

                        if ($idPosicao == 'I' && ($keyLinha <= $interFim - 1 && $keyLinha >= $interIni - 1)) {
                            $nrLinha = $paramLinha->getNrlinhafontedados();
                            $arrLinha = explode($separador, $linha);
                            $posSeparador = strpos($linha, $separador);

                            self::fillDataSourceSeparador($paramColunas[$nrLinha], $keyLinha, $linha, $dataSource,
                                $nrLinha, $posSeparador, $arrLinha, $interIni, $interFim);
                        }
                    }
                }
            }
        } else {
            $paramLinhas  = $params['LINHAS'];
            $paramColunas = $params['COLUNAS'];

            foreach($fileLinhas as $keyLinha => $linha) {
                //Primeira Linha
                if($keyLinha == 0) {
                    foreach ($paramLinhas as $paramLinha) {
                        if($paramLinha->getIdposicaolinha() == 'P') {
                            self::fillDataSource($paramColunas[$paramLinha->getNrlinhafontedados()], $keyLinha, $linha, $dataSource, $paramLinha->getNrlinhafontedados());
                        }
                    }
                }
                //Última Linha
                else if($keyLinha == (count($fileLinhas) - 1)) {
                    foreach ($paramLinhas as $paramLinha) {
                        if($paramLinha->getIdposicaolinha() == 'U') {
                            self::fillDataSource($paramColunas[$paramLinha->getNrlinhafontedados()], $keyLinha, $linha, $dataSource, $paramLinha->getNrlinhafontedados());
                        }
                    }
                }
                //Intervalo de Linhas
                else {
                    foreach ($paramLinhas as $paramLinha) {
                        if($paramLinha->getIdposicaolinha() == 'I' && ($keyLinha <= $paramLinha->getNrintervalofinal() - 1 &&
                                                                       $keyLinha >= $paramLinha->getNrintervaloinicial() - 1)) {
                            self::fillDataSource($paramColunas[$paramLinha->getNrlinhafontedados()], $keyLinha, $linha, $dataSource,
                                $paramLinha->getNrlinhafontedados(), $paramLinha->getNrintervaloinicial(), $paramLinha->getNrintervalofinal());
                        }
                    }
                }
            }
        }

        $dataSource = self::organizeDataSource($dataSource);

        return $dataSource;
    }

    private static function validarXLS($fileName, $params)
    {
        $diretorio = realpath('../uploads');
        $file      = $diretorio.DIRECTORY_SEPARATOR.$fileName;

        $xlsFile    = new ExcelUtils($file);
        $xlsLines   = $xlsFile->buildDataIntoJson(true);
        $dataSource = array();

        foreach ($xlsLines as $keyLines => $lines) {
            foreach ($lines as $keyLine => $value) {
                $tipo     = $params[$keyLine]['TIPO'];
                $mask     = $params[$keyLine]['MASK'];
                $required = $params[$keyLine]['REQUIRED'];

                $valueColum = self::validateValue($tipo, trim($value), $mask);

                if(($valueColum == '' || is_null($valueColum) || $valueColum == false) && $required == 'S') {
                    $dataSource[$keyLines][$params[$keyLine]['FIELD']] = 'NOTFOUND';
                } else {
                    $dataSource[$keyLines][$params[$keyLine]['FIELD']] = $valueColum;
                }
            }
        }

        return $dataSource;
    }

    private static function validarTIC($fileName, $params)
    {
        $diretorio = realpath('../uploads');
        $file      = $diretorio.DIRECTORY_SEPARATOR.$fileName;

        $fileLinhas = file($file, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);

        //Codificação necessária dos caracteres especiais (ç, á, ...).
        foreach ($fileLinhas as &$fileLinha) {
            $fileLinha = utf8_encode($fileLinha);
        }

        $dataSource   = array();
        $paramLinhas  = $params['LINHAS'];
        $paramColunas = $params['COLUNAS'];

        foreach($fileLinhas as $keyLinha => $linha) {
            $aux = false;

            foreach ($paramLinhas as $paramLinha) {
                $interIni   = $paramLinha->getNrintervaloinicial();
                $interFim   = $paramLinha->getNrintervalofinal();
                $nrLinha    = $paramLinha->getNrlinhafontedados();
                $vrIdColuna = $paramLinha->getVridlinhaporcoluna();
                $vrIdFile   = substr($linha, $interIni - 1, ($interFim - $interIni) + 1);

                if ($vrIdFile == $vrIdColuna) {
                    $aux = true;
                    break;
                }
            }

            if ($aux) {
                self::fillDataSourceTIC($paramColunas[$nrLinha], $keyLinha, $linha, $dataSource, $nrLinha, $vrIdColuna);
            } else {
                $dataSource[$keyLinha]['FIELD0'] = 'A linha: ' . '"' . $linha . '"' . ' não foi reconhecida.';
            }
        }

        $dataSource = self::organizeDataSourceTIC($dataSource);

        return $dataSource;
    }

    /*
     * Preenche o array dataSource, sendo que cada posição sua é um array com os dados das colunas
     */
    private static function fillDataSource($paramColunas, $keyLinha, $linha, &$dataSource, $nrLinha, $interLinhaIni = null, $interLinhaFim = null)
    {
        for($i = 0; $i < count($paramColunas); $i++) {
            $tipo     = $paramColunas[$i]['TIPO'];
            $mask     = $paramColunas[$i]['MASK'];
            $required = $paramColunas[$i]['REQUIRED'];

            $valueColum = trim(substr($linha, $paramColunas[$i]['POS_INI'] - 1, ($paramColunas[$i]['POS_FIM'] - $paramColunas[$i]['POS_INI']) + 1));
            $valueColum = self::validateValue($tipo, $valueColum, $mask);

            if(($valueColum == '' || is_null($valueColum) || $valueColum == false) && $required == 'S') {
                $valueColum = 'NOTFOUND';
            }

            $dataSource[$keyLinha][$paramColunas[$i]['FIELD']] = $valueColum;

            //Em sua última interação insere também as posições iniciais e finais, que só serão passadas se for intervalo de linhas
            if($i == count($paramColunas) - 1 && $interLinhaIni) {
                $dataSource[$keyLinha]['POSICAO_INI'] = $interLinhaIni;
                $dataSource[$keyLinha]['POSICAO_FIM'] = $interLinhaFim;
            }
        }

        $dataSource[$keyLinha]['NRLINHA'] = $nrLinha;
    }

    private static function fillDataSourceSeparador($params, $keyLinha, $linha, &$dataSource, $nrLinha, $posSeparador, $arrLinha, $interLinhaIni = null, $interLinhaFim = null)
    {
        if(!$posSeparador) {
            foreach($params as $param) {
                if ($param['FIELD'] == 'FIELD0') {
                    $dataSource[$keyLinha][$param['FIELD']] = 'Separador não encontrado na linha: ' . $linha;
                } else {
                    $dataSource[$keyLinha][$param['FIELD']] = 'NOTFOUND';
                }
            }
        } else {
            foreach ($arrLinha as $keyColum => $coluna) {
                $tipo     = $params[$keyColum]['TIPO'];
                $mask     = $params[$keyColum]['MASK'];
                $required = $params[$keyColum]['REQUIRED'];

                $valueColum = self::validateValue($tipo, trim($coluna), $mask);

                if ($valueColum == '' && $required == 'S') {
                    $dataSource[$keyLinha][$params[$keyColum]['FIELD']] = 'NOTFOUND';
                } else {
                    $dataSource[$keyLinha][$params[$keyColum]['FIELD']] = $valueColum;
                }

                if($keyColum == count($arrLinha) - 1 && $interLinhaIni) {
                    $dataSource[$keyLinha]['POSICAO_INI'] = $interLinhaIni;
                    $dataSource[$keyLinha]['POSICAO_FIM'] = $interLinhaFim;
                }
            }
        }

        $dataSource[$keyLinha]['NRLINHA'] = $nrLinha;
    }

    private static function fillDataSourceTIC($paramColunas, $keyLinha, $linha, &$dataSource, $nrLinha, $vrIdColuna)
    {
        for($i = 0; $i < count($paramColunas); $i++) {
            $posIni   = $paramColunas[$i]['POS_INI'];
            $posFim   = $paramColunas[$i]['POS_FIM'];
            $field    = $paramColunas[$i]['FIELD'];
            $tipo     = $paramColunas[$i]['TIPO'];
            $mask     = $paramColunas[$i]['MASK'];
            $required = $paramColunas[$i]['REQUIRED'];

            $valueColum = trim(substr($linha, $posIni - 1, ($posFim - $posIni) + 1));
            $valueColum = self::validateValue($tipo, $valueColum, $mask);

            if(($valueColum == '' || is_null($valueColum) || $valueColum == false) && $required == 'S') {
                $valueColum = 'NOTFOUND';
            }

            $dataSource[$keyLinha][$field] = $valueColum;
        }

        $dataSource[$keyLinha]['NRLINHA'] = $nrLinha;
        $dataSource[$keyLinha]['VRIDCOLUNA'] = $vrIdColuna;
    }

    /*
     * Organiza o dataSource, fazendo a junção das linhas que fazem parte de um mesmo intervalo em um array separado,
     * para serem mostradas juntas no grid de validação.
     */
    private static function organizeDataSource($dataSource)
    {
        $orgDataSource = array();
        $auxDataSource = array();
        $posicaoIniAnt = 0;

        foreach ($dataSource as $data) {
            if(isset($data['POSICAO_INI'])) {
                if($posicaoIniAnt == 0) {
                    $auxDataSource[] = $data;
                    $posicaoIniAnt   = $data['POSICAO_INI'];
                } else if($posicaoIniAnt == $data['POSICAO_INI']) {
                    $auxDataSource[] = $data;
                } else {
                    $orgDataSource[] = $auxDataSource;
                    $auxDataSource   = array();
                    $posicaoIniAnt   = $data['POSICAO_INI'];
                    $auxDataSource[] = $data;
                }
            } else {
                if($posicaoIniAnt != 0) {
                    $orgDataSource[] = $auxDataSource;
                    $orgDataSource[] = $data;
                } else {
                    $orgDataSource[] = $data;
                }
            }
        }

        return $orgDataSource;
    }

    private static function organizeDataSourceTIC($dataSource)
    {
        $orgDataSource = array();
        $auxDataSource = array();
        $auxVrIdColuna = '';

        foreach ($dataSource as $key => $data) {
            if(isset($data['VRIDCOLUNA'])) {
                if($auxVrIdColuna == '') {
                    $auxDataSource[] = $data;
                    $auxVrIdColuna   = $data['VRIDCOLUNA'];
                } else if($auxVrIdColuna == $data['VRIDCOLUNA']) {
                    $auxDataSource[] = $data;
                } else {
                    $orgDataSource[] = $auxDataSource;
                    $auxDataSource   = array();
                    $auxVrIdColuna   = $data['VRIDCOLUNA'];
                    $auxDataSource[] = $data;

                    if($key == count($dataSource) - 1) {
                        $orgDataSource[] = $auxDataSource;
                    }
                }
            } else {
                $orgDataSource[] = $data;
            }
        }

        return $orgDataSource;
    }

    public static function getDataSourceParamSQL($paramsSQL)
    {
        $dataSource = array();

        foreach ($paramsSQL as $key => $param) {
            $dataSource[$key]['PARAM'] = $param['LABEL'];
            $dataSource[$key]['VALUE'] = '';
            $dataSource[$key]['TYPE']  = $param['TIPO'];
        }

        return $dataSource;
    }

    /**
     * @param $params -> Array com todos os parâmetros do comando sql
     * @param $dataSet -> Array de arrays com parâmetros e valores alterados pelo usuário.
     * @return array
     */
    public static function getParamsDataSetSQL($params, $dataSet)
    {
        $paramsRet = array();

        foreach ($dataSet as $row) {
            $paramsRet[$row['PARAM']] = $row['VALUE'];
        }

        foreach ($params as $param) {
            if(!isset($paramsRet[$param['LABEL']])) {
                $paramsRet[$param['LABEL']] = null;
            }
        }

        return $paramsRet;
    }

    public static function executeComandoSQL($comandoSQL, $parametros)
    {
        try{
            $fetch = \Setforms2\Shared\Connection::getInstance()->fetchAll($comandoSQL, $parametros);

            if(count($fetch) == 0) {
                return false;
            }

            return $fetch;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $arrayFetch -> array retornado pela execução de um comando sql.
     * @return array
     */
    public static function getDataSourceFonteSQL($arrayFetch)
    {
        $dataSource = array();

        for($i = 0; $i < count($arrayFetch); $i++) {
            $aux = 0;
            foreach ($arrayFetch[$i] as $colum) {
                $dataSource[$i]['FIELD'.$aux] = $colum;
                $aux++;
            }
        }

        return $dataSource;
    }

    public static function getTxComandoSqlLinha($nrFonte, $nrOrg)
    {
        $linha = Linhafontedados::findOneByNrfontedadosAndNrorg($nrFonte, $nrOrg);

        if(is_object($linha)) {
            return $linha->getTxcomandosql();
        }

        return false;
    }

    public static function getColumsComandoSQL($nrFonteDados, $nrOrg)
    {
        $linha = Linhafontedados::findOneByNrfontedadosAndNrorg($nrFonteDados, $nrOrg);
        $columsRet = array();

        if(is_object($linha)) {
            $params  = array($linha->getNrlinhafontedados(), $nrOrg);
            $where   = 'NRLINHAFONTEDADOS = ? AND NRORG = ?';
            $orderBy = 'NRPOSICAOINICIAL';

            $colums = Colunafontedados::getObjectsByWhere($where, $params, $orderBy);

            foreach($colums as $key => $colum) {
                $columsRet[] = array('LABEL' => $colum->getDscolunafontedados(),
                                     'FIELD' => 'FIELD'.$key);
            }
        }

        return $columsRet;
    }

    public static function deleteFile($fileName, $filePath)
    {
        $objDir = glob($filePath.'/*');

        foreach ($objDir as $file) {
            if(basename($file) == $fileName) {
                unlink($filePath.DIRECTORY_SEPARATOR.$fileName);
            }
        }
    }

    public static function validaExtensaoArquivo($fileName, $idFonteDados)
    {
        $extensao = pathinfo($fileName, PATHINFO_EXTENSION);

        if($idFonteDados == 'TPL' || $idFonteDados == 'TIC') {
            if($extensao != 'txt') {
                return false;
            }

        } else if($idFonteDados == 'XLS') {
            if($extensao != 'xls') {
                return false;
            }
        }

        return true;
    }

    private static function validateValue($type, $value, $mask)
    {
        $valueReturn = '';

        switch($type) {
            case 'H':
                $posHora    = strpos(strtolower($mask), 'hh');
                $posMinuto  = strpos(strtolower($mask), 'mi');
                $posSegundo = strpos(strtolower($mask), 'ss');

                $hora    = substr($value, $posHora, 2);
                $minuto  = substr($value, $posMinuto, 2);
                $segundo = '';

                if ($posSegundo) {
                    $segundo = substr($value, $posSegundo, 2);
                }

                if ($hora && $minuto && preg_match("/^[0123456789:]+$/", $hora . $minuto . $segundo)) {
                    $valueReturn = $hora . ':' . $minuto;

                    if ($posSegundo) {
                        if($segundo != '') {
                            $valueReturn .= ':' . $segundo;
                        } else if ($value != '') {
                            $valueReturn = 'Hora Inválida';
                        }
                    }
                } else if ($value != '') {
                    $valueReturn = 'Hora Inválida';
                }

                break;
            case 'D':
                $posDia = strpos(strtolower($mask), 'dd');
                $posMes = strpos(strtolower($mask), 'mm');
                $posAno = strpos(strtolower($mask), 'yyyy');

                $dia = substr($value, $posDia, 2);
                $mes = substr($value, $posMes, 2);
                $ano = substr($value, $posAno, 4);
                $qtdCharAno = 4;

                if($posAno == false) {
                    $posAno = strpos(strtolower($mask), 'yy');
                    $ano    = substr($value, $posAno, 2);
                    $qtdCharAno = 2;
                }

                $strData  = $dia . '/' . $mes . '/' . $ano;
                $dtFormat = false;

                if($qtdCharAno == 4 && (strlen($ano) == $qtdCharAno)) {
                    $data     = DateUtil::getDataDeString($strData, 'd/m/Y');
                    $dtFormat = date_format($data, 'd/m/Y');
                } else if(strlen($ano) == $qtdCharAno){
                    $data     = DateUtil::getDataDeString($strData, 'd/m/y');
                    $dtFormat = date_format($data, 'd/m/y');
                }

                if ($dtFormat) {
                    $valueReturn = $dtFormat;
                } else if($value != '') {
                    $valueReturn = 'Data Inválida';
                }

                break;
            case 'F':
                //Lista de caracteres aceitos em um número Float
                if(preg_match("/^[0123456789,.]+$/", $value)) {
                    $posVirgula = strpos($mask, ',');
                    $posPonto   = strpos($mask, '.');
                    $value = str_replace(array('.', ','), array('', ''), $value);

                    if($posVirgula && $posPonto) {
                        //Quantidade de casas decimais definidas na máscara
                        $decimalMask = substr($mask, $posVirgula + 1);
                        $tamDec      = strlen($decimalMask);
                        //Parte decimal do valor, de acordo com a da máscara
                        $decimal     = substr($value, -$tamDec);
                        //Monta o valor com o separador de decimal
                        $newValue    = substr($value, 0, strlen($value) - $tamDec).'.'.$decimal;
                        $valueReturn = number_format($newValue, $tamDec, ',', '.');
                    } else if($posVirgula && $posPonto == false) {
                        $decimalMask = substr($mask, $posVirgula + 1);
                        $tamDec      = strlen($decimalMask);
                        $decimal     = substr($value, -$tamDec);
                        $newValue    = substr($value, 0, strlen($value) - $tamDec).'.'.$decimal;
                        $valueReturn = number_format($newValue, $tamDec, ',', '');
                    } else if($posVirgula == false && $posPonto) {
                        $valueReturn = number_format($value, 0, '', '.');
                    }

                    if(!$valueReturn && $value != '') {
                        $valueReturn = 'Valor Inválido';
                    }
                } else if($value != '') {
                    $valueReturn = 'Valor Inválido';
                }

                break;
            case 'N':
                if(preg_match("/^[0-9]+$/", $value)) {
                    $valueReturn = $value;
                } else if($value != '') {
                    $valueReturn = 'Número Inválido';
                }

                break;
            default:
                $valueReturn = $value;
                break;
        }

        return $valueReturn;
    }

    public static function validateMask($idTipoColuna, $maskColuna)
    {
        $validacao = true;

        switch($idTipoColuna) {
            //Verifica se na máscara existe as identicações de hora(hh) e minuto(mi)
            case 'H':
                $hora   = strpos(strtolower($maskColuna), 'hh');
                $minuto = strpos(strtolower($maskColuna), 'mi');

                if($hora === false || $minuto === false) {
                    $validacao = false;
                }

                break;
            //Verifica se na máscara existe as identicações de dia(dd), mês(mm) e ano(yy)
            case 'D':
                $dia = strpos(strtolower($maskColuna), 'dd');
                $mes = strpos(strtolower($maskColuna), 'mm');
                $ano = strpos(strtolower($maskColuna), 'yy');

                if($dia === false || $mes === false || $ano === false) {
                    $validacao = false;
                }

                break;
            //Verifica se os caracteres da máscara fazem parte de um número float e se existe pelo menos um
            //separador de decimal ou milhar (, ou .)
            case 'F':
                if(!preg_match("/^[0123456789,.]+$/", $maskColuna)) {
                    $validacao = false;
                } else if(!strpos($maskColuna, ',') && !strpos($maskColuna, '.')) {
                    $validacao = false;
                }

                break;
            default:
                $validacao = true;
                break;
        }

        return $validacao;
    }

    private static function validateValueParamSql($type, $value)
    {
        $valueReturn = true;

        switch($type) {
            case 'H':
                //Lista de caracteres aceitos em um Tipo Hora
                if (preg_match("/^[0123456789:]+$/", $value) && strlen($value) >= 0) {
                    $valueReturn = true;
                } else if($value != '') {
                    $valueReturn = 'Hora Inválida';
                }

                break;
            case 'D':
                $data = DateUtil::getDataDeString($value, 'd/m/Y');

                if($data) {
                    $valueReturn = true;
                } else if($value != '') {
                    $valueReturn = 'Data Inválida';
                }

                break;
            case 'F':
                //Lista de caracteres aceitos em um Tipo Float
                if(preg_match("/^[0123456789,.]+$/", $value)) {
                    $valueReturn = true;
                } else if($value != '') {
                    $valueReturn = 'Valor Inválido';
                }

                break;
            case 'N':
                //Caracteres aceitos em um Tipo Número.
                if(preg_match("/^[0-9]+$/", $value)) {
                    $valueReturn = true;
                } else if($value != '') {
                    $valueReturn = 'Número Inválido';
                }

                break;
            default:
                $valueReturn = true;
                break;
        }

        return $valueReturn;
    }

    public static function validateParamsSql($dataSet)
    {
        $message = '';

        foreach ($dataSet as $row) {
            $type  = $row['TYPE'];
            $value = $row['VALUE'];
            $param = $row['PARAM'];

            $validacao = self::validateValueParamSql($type, $value);

            if($validacao !== true) {
                $message .= tk_get_message_by_name('msg_ger00001_003_ValidacaoParamSql', array($validacao, $param));
            }
        }

        if($message != '') {
            $message = rtrim($message, '; <br/>').'.';
        }

        return $message;
    }
}