CREATE OR REPLACE PROCEDURE PRE_VALIDACAO_REINF(
   P_CDIMPOSTO    IN IMPOSTO.CDIMPOSTO%TYPE,
   P_CDEMPRESA    IN EMPRESA.CDEMPRESA%TYPE,
   P_NRSEQUMAPAPU IN MAPAAPURAC.NRSEQUMAPAPU%TYPE,
   P_INSCRICAO    IN NOTAFISCAL.CDINSCESTA%TYPE,
   P_IDORGARRECAD IN IMPOSTO.IDORGARRECAD%TYPE,
   P_CDLIVROMAG   IN LIVROMAGNETICO.CDLIVROMAG%TYPE)
AS
   V_TOTALITIMP   NUMBER;
   V_TOTALIMPNF   NUMBER;
   V_VRCONTITIMP  NUMBER;
   V_SEQERRO      NUMBER;
   V_STSEQERRO    VARCHAR2(3);
   V_DTINIMAPA    DATE;
   V_DTFIMMAPA    DATE;
   V_SEQUENCIAL   NUMBER;
   V_DIFERENCA    FLOAT;
   V_DSDIRCERTIFICADO VARCHAR2(500);
   V_IDINDCPRB   VARCHAR2(01);
   V_CDPRODSERV  VARCHAR2(50);
BEGIN
   DELETE FROM ERRONFPREVALID
    WHERE (CDFILIAL, NRLANCTONF) IN (SELECT CDFILIAL, NRLANCTONF
                                       FROM IMPOSTONF
                                      WHERE (NRSEQUMAPAPU = P_NRSEQUMAPAPU OR NRSEQMAPOIMP = P_NRSEQUMAPAPU)
                                        AND DSTIPOVALIDA = 'REINF');

   DELETE FROM NFPREVALIDFIS
    WHERE NRSEQUMAPAPUNFP = P_NRSEQUMAPAPU;

   BEGIN
      SELECT DSDIRCERTIFICADO, IDINDCPRB
        INTO V_DSDIRCERTIFICADO, V_IDINDCPRB
        FROM PARAMEMPREINF
       WHERE CDEMPRESA = P_CDEMPRESA;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_DSDIRCERTIFICADO := NULL;
         V_IDINDCPRB := NULL;
   END;

   FOR C_NF IN (SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.NRNOTAFISC, N.IDENTRSAIDOP, N.NRACESSONFE,
                       N.VRNOTAFISC, N.DTEMISSAO, N.DTENTRSAID, N.CDFORNECED, N.CDCLIENTE, N.CDFILIEMIT, N.CDFILIRECE,
                       N.CDSERIEENTR, N.CDSERIE, N.CDINSCMUNI
                  FROM NOTAFISCAL N, IMPOSTONF I, IMPOSTO IM
                 WHERE N.CDFILIAL      = I.CDFILIAL
                   AND N.NRLANCTONF    = I.NRLANCTONF
                   AND N.IDSISTEMA     <> 'FIS'
                   AND N.IDSISTEMA     <> 'TMP'
                   AND ((N.CDINSCESTA  = P_INSCRICAO AND P_IDORGARRECAD  = 'E') OR
                        (N.CDINSCMUNI  = P_INSCRICAO AND P_IDORGARRECAD  = 'M') OR
                        (P_INSCRICAO   IS NULL       AND NVL(P_IDORGARRECAD,'F') = 'F'))
                   AND IM.CDIMPOSTONFE = P_CDIMPOSTO
                   AND IM.CDIMPOSTO    = I.CDIMPOSTO
                   AND (I.NRSEQUMAPAPU = P_NRSEQUMAPAPU OR I.NRSEQMAPOIMP = P_NRSEQUMAPAPU)
                   AND I.CDFISCALNF    = '1'
                 ORDER BY N.CDFILIAL, N.NRLANCTONF) LOOP

      INSERT INTO NFPREVALIDFIS
        (CDFILIAL, NRLANCTONF, DTEMISSAONFPRE, DTENTRSAIDNFPRE, IDENTRSAIDNFPRE, NRNOTAFISCNFPRE, VRNOTAFISCNFPRE,
         NRSEQUMAPAPUNFP, CDFORNECEDNFPRE, CDCLIENTENFPRE, CDFILIEMITNFPRE, CDFILIRECENFPRE)
      VALUES
        (C_NF.CDFILIAL, C_NF.NRLANCTONF, C_NF.DTEMISSAO, C_NF.DTENTRSAID, C_NF.IDENTRSAIDOP, C_NF.NRNOTAFISC, C_NF.VRNOTAFISC,
         P_NRSEQUMAPAPU, C_NF.CDFORNECED, C_NF.CDCLIENTE, C_NF.CDFILIEMIT, C_NF.CDFILIRECE);
      V_SEQERRO := 0;

      BEGIN
         SELECT DTINIPERIODO, DTFIMPERIODO
           INTO V_DTINIMAPA, V_DTFIMMAPA
           FROM MAPAAPURAC
          WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_DTINIMAPA   := NULL;
            V_DTFIMMAPA   := NULL;
      END;

      IF C_NF.IDENTRSAIDOP = 'E' THEN
         BEGIN
            SELECT IDINDCPRB
              INTO V_IDINDCPRB
              FROM FORNECEDOR
             WHERE CDFORNECED = C_NF.CDFORNECED;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_IDINDCPRB := NULL;
         END;
      END IF;

      IF V_DSDIRCERTIFICADO IS NULL THEN
         V_SEQERRO   := V_SEQERRO + 1;
         V_STSEQERRO := LPAD(TO_CHAR(V_SEQERRO), 3, '0');
         INSERT INTO ERRONFPREVALID
           (CDFILIAL, NRLANCTONF, NRSEQERRONFPRE, DSTIPOVALIDA, DSERRONFPRE, IDERRONFPRE, IDERROAVISNFPRE)
         VALUES
           (C_NF.CDFILIAL, C_NF.NRLANCTONF, V_STSEQERRO, 'REINF',
            'Nao foi identificado parametrizacao do certificado digital para transmissao do Reinf.', 'CERTIFICADO', 'A');
      END IF;

      IF C_NF.DTEMISSAO > V_DTFIMMAPA THEN
         V_SEQERRO   := V_SEQERRO + 1;
         V_STSEQERRO := LPAD(TO_CHAR(V_SEQERRO), 3, '0');
         INSERT INTO ERRONFPREVALID
            (CDFILIAL, NRLANCTONF, NRSEQERRONFPRE, DSTIPOVALIDA, DSERRONFPRE, IDERRONFPRE, IDERROAVISNFPRE)
         VALUES
             (C_NF.CDFILIAL, C_NF.NRLANCTONF, V_STSEQERRO, 'REINF',
             'A data de emisso da nota deve ser menor ou igual  data final do mapa de apurao.', 'MODELO', 'A');
      END IF;

      IF (C_NF.DTENTRSAID > V_DTFIMMAPA) AND (C_NF.CDMODDOCFISC NOT IN ('06','29','28')) THEN
          V_SEQERRO   := V_SEQERRO + 1;
          V_STSEQERRO := LPAD(TO_CHAR(V_SEQERRO), 3, '0');
          INSERT INTO ERRONFPREVALID
             (CDFILIAL, NRLANCTONF, NRSEQERRONFPRE, DSTIPOVALIDA, DSERRONFPRE, IDERRONFPRE, IDERROAVISNFPRE)
          VALUES
             (C_NF.CDFILIAL, C_NF.NRLANCTONF, V_STSEQERRO, 'REINF',
             'A data de entrada da nota deve ser menor ou igual  data final do mapa de apurao', 'MODELO', 'A');
      END IF;

      BEGIN
         IF TO_NUMBER(C_NF.NRNOTAFISC) = 0 THEN
            V_SEQERRO   := V_SEQERRO + 1;
            V_STSEQERRO := LPAD(TO_CHAR(V_SEQERRO), 3, '0');
            INSERT INTO ERRONFPREVALID
               (CDFILIAL, NRLANCTONF, NRSEQERRONFPRE, DSTIPOVALIDA, DSERRONFPRE, IDERRONFPRE, IDERROAVISNFPRE)
            VALUES
               (C_NF.CDFILIAL, C_NF.NRLANCTONF, V_STSEQERRO, 'REINF',
               'Nmero da nota fiscal deve ser maior que zero', 'NUMERO', 'E');
         END IF;
      EXCEPTION
        WHEN VALUE_ERROR THEN
           V_SEQERRO   := V_SEQERRO + 1;
           V_STSEQERRO := LPAD(TO_CHAR(V_SEQERRO), 3, '0');
           INSERT INTO ERRONFPREVALID
              (CDFILIAL, NRLANCTONF, NRSEQERRONFPRE, DSTIPOVALIDA, DSERRONFPRE, IDERRONFPRE, IDERROAVISNFPRE)
           VALUES
              (C_NF.CDFILIAL, C_NF.NRLANCTONF, V_STSEQERRO, 'REINF',
              'Nmero da nota fiscal possui caracteres invlidos', 'NUMERO', 'E');
      END;

      FOR C_ITEMNF IN (SELECT CDSITUTRIBIT, CDFISCOPERIT, CDFISCALIT, VRBASECALCIT, VRALIQITEM, VRIMPOSTOIT,
                              I.NRSEQITPROD, NRITPROIMPOS, P.CDPRODUTO, I.VRCONTITIMP, I.VRPERREDBCIT,
                              PP.CDARVPROD, PP.NMPRODUTO
                         FROM ITEMPROIMPOS I, ITEMPRODUTO P, PRODUTO PP, IMPOSTO IM, NOTAFISCAL N
                        WHERE I.CDFILIAL      = C_NF.CDFILIAL
                          AND I.NRLANCTONF    = C_NF.NRLANCTONF
                          AND I.NRLANCTONF    = N.NRLANCTONF
                          AND I.CDFILIAL      = N.CDFILIAL
                          AND N.IDSISTEMA     <> 'FIS'
                          AND N.IDSISTEMA     <> 'TMP'
                          AND IM.CDIMPOSTONFE = P_CDIMPOSTO
                          AND IM.CDIMPOSTO    = I.CDIMPOSTO
                          AND P.CDFILIAL      = I.CDFILIAL
                          AND P.NRLANCTONF    = I.NRLANCTONF
                          AND P.NRSEQITPROD   = I.NRSEQITPROD
                          AND PP.CDPRODUTO    = P.CDPRODUTO
                        ORDER BY I.CDIMPOSTO, I.NRSEQITPROD) LOOP

         V_SEQUENCIAL := NVL(V_SEQUENCIAL,0)+1;
         IF C_ITEMNF.CDFISCALIT = '1' THEN
            IF C_ITEMNF.VRBASECALCIT = 0 THEN
               V_SEQERRO   := V_SEQERRO + 1;
               V_STSEQERRO := LPAD(TO_CHAR(V_SEQERRO), 3, '0');
               INSERT INTO ERRONFPREVALID
                  (CDFILIAL, NRLANCTONF, NRSEQERRONFPRE, DSTIPOVALIDA, DSERRONFPRE, IDERRONFPRE, IDERROAVISNFPRE)
               VALUES
                  (C_NF.CDFILIAL, C_NF.NRLANCTONF, V_STSEQERRO, 'REINF',
                  'O valor da base de clculo no pode ser zero para itens de cdigo fiscal 1. (Item: '||V_SEQUENCIAL||')', 'CF', 'E');
            END IF;

            IF C_ITEMNF.VRCONTITIMP < 0 THEN
               V_SEQERRO   := V_SEQERRO + 1;
               V_STSEQERRO := LPAD(TO_CHAR(V_SEQERRO), 3, '0');
               INSERT INTO ERRONFPREVALID
                  (CDFILIAL, NRLANCTONF, NRSEQERRONFPRE, DSTIPOVALIDA, DSERRONFPRE, IDERRONFPRE, IDERROAVISNFPRE)
               VALUES
                  (C_NF.CDFILIAL, C_NF.NRLANCTONF, V_STSEQERRO, 'REINF',
                   'Item com valor contbil negativo. (Item: '||V_SEQUENCIAL||')', 'VR_CONTABIL', 'E');
            END IF;

            IF V_IDINDCPRB = '0' THEN
               IF C_ITEMNF.VRALIQITEM <> 11 THEN
                  V_SEQERRO   := V_SEQERRO + 1;
                  V_STSEQERRO := LPAD(TO_CHAR(V_SEQERRO), 3, '0');
                  INSERT INTO ERRONFPREVALID
                     (CDFILIAL, NRLANCTONF, NRSEQERRONFPRE, DSTIPOVALIDA, DSERRONFPRE, IDERRONFPRE, IDERROAVISNFPRE)
                  VALUES
                     (C_NF.CDFILIAL, C_NF.NRLANCTONF, V_STSEQERRO, 'REINF',
                      'Valor de alquota invlido ('||C_ITEMNF.VRALIQITEM||') para o CPRB parametrizado. (Item: '||V_SEQUENCIAL||').', 'VR_ALIQUOTA', 'A');
               END IF;
            ELSE
               IF C_ITEMNF.VRALIQITEM <> 3.5 THEN
                  V_SEQERRO   := V_SEQERRO + 1;
                  V_STSEQERRO := LPAD(TO_CHAR(V_SEQERRO), 3, '0');
                  INSERT INTO ERRONFPREVALID
                     (CDFILIAL, NRLANCTONF, NRSEQERRONFPRE, DSTIPOVALIDA, DSERRONFPRE, IDERRONFPRE, IDERROAVISNFPRE)
                  VALUES
                     (C_NF.CDFILIAL, C_NF.NRLANCTONF, V_STSEQERRO, 'REINF',
                      'Valor de alquota invlido ('||C_ITEMNF.VRALIQITEM||') para o CPRB parametrizado. (Item: '||V_SEQUENCIAL||').', 'VR_ALIQUOTA', 'A');
               END IF;
            END IF;
         END IF;

         BEGIN
            SELECT MAX(CDPRODSERV)
              INTO V_CDPRODSERV
              FROM PARAMMUNISERV
             WHERE CDEMPRESA  = P_CDEMPRESA
               AND CDINSCMUNI = C_NF.CDINSCMUNI
               AND CDPRODSERV = C_ITEMNF.CDPRODUTO;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_CDPRODSERV := NULL;
         END;

         IF V_CDPRODSERV IS NULL THEN
            V_SEQERRO   := V_SEQERRO + 1;
            V_STSEQERRO := LPAD(TO_CHAR(V_SEQERRO), 3, '0');
            INSERT INTO ERRONFPREVALID
               (CDFILIAL, NRLANCTONF, NRSEQERRONFPRE, DSTIPOVALIDA, DSERRONFPRE, IDERRONFPRE, IDERROAVISNFPRE)
            VALUES
               (C_NF.CDFILIAL, C_NF.NRLANCTONF, V_STSEQERRO, 'REINF',
                'No foi encontrado parametrizacao de cdigo de exportacao para o servico: '||C_ITEMNF.CDARVPROD ||' - '|| C_ITEMNF.NMPRODUTO ||'.', 'PR_SERVICO', 'E');
         END IF;
         V_CDPRODSERV := NULL;
      END LOOP;

      V_SEQUENCIAL := 0;
      BEGIN
         SELECT ROUND(SUM(VRCONTITIMP),2)
           INTO V_TOTALITIMP
           FROM ITEMPROIMPOS I, IMPOSTO IM
          WHERE CDFILIAL   = C_NF.CDFILIAL
            AND NRLANCTONF = C_NF.NRLANCTONF
            AND IM.CDIMPOSTONFE = P_CDIMPOSTO
            AND NVL(I.NRITPROIMPOS,'3') <> '3'
            AND IM.CDIMPOSTO = I.CDIMPOSTO;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_TOTALITIMP := 0;
      END;
      BEGIN
         SELECT ROUND(SUM(VRCONTABILNF),2)
           INTO V_TOTALIMPNF
           FROM IMPOSTONF I, IMPOSTO IM
          WHERE CDFILIAL   = C_NF.CDFILIAL
            AND NRLANCTONF = C_NF.NRLANCTONF
            AND IM.CDIMPOSTONFE = P_CDIMPOSTO
            AND NVL(I.IDIMPOSTONF,'3') <> '3'
            AND IM.CDIMPOSTO = I.CDIMPOSTO;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_TOTALIMPNF := 0;
      END;
      IF V_TOTALITIMP <> V_TOTALIMPNF THEN
         V_SEQERRO   := V_SEQERRO + 1;
         V_STSEQERRO := LPAD(TO_CHAR(V_SEQERRO), 3, '0');
         INSERT INTO ERRONFPREVALID
            (CDFILIAL, NRLANCTONF, NRSEQERRONFPRE, DSTIPOVALIDA, DSERRONFPRE, IDERRONFPRE, IDERROAVISNFPRE)
         VALUES
            (C_NF.CDFILIAL, C_NF.NRLANCTONF, V_STSEQERRO, 'REINF',
            'Valor contbil dos itens da nota  diferente do valor contbil do analtico por imposto. (Valor do analtico R$'||V_TOTALITIMP||'. Valor dos itens da nota R$'||V_TOTALIMPNF||')', 'VR_CONTABIL', 'E');
      END IF;

      IF V_TOTALIMPNF < 0 THEN
         V_SEQERRO   := V_SEQERRO + 1;
         V_STSEQERRO := LPAD(TO_CHAR(V_SEQERRO), 3, '0');
         INSERT INTO ERRONFPREVALID
            (CDFILIAL, NRLANCTONF, NRSEQERRONFPRE, DSTIPOVALIDA, DSERRONFPRE, IDERRONFPRE, IDERROAVISNFPRE)
         VALUES
            (C_NF.CDFILIAL, C_NF.NRLANCTONF, V_STSEQERRO, 'REINF',
            'Nota com valor contbil negativo', 'VR_CONTABIL', 'E');
      END IF;
   END LOOP;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('PRE_VALIDACAO_REINF','6.05.000'); END;
/